package logic

import (
	"encoding/json"
	"errors"
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"fusenapi/utils/format"
	"fusenapi/utils/image"
	"fusenapi/utils/s3url_to_s3id"
	"gorm.io/gorm"
	"sort"
	"strings"

	"context"

	"fusenapi/server/product/internal/svc"
	"fusenapi/server/product/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetTagProductListLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewGetTagProductListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetTagProductListLogic {
	return &GetTagProductListLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *GetTagProductListLogic) GetTagProductList(req *types.GetTagProductListReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	//查询用户信息(不用判断存在)
	user, err := l.svcCtx.AllModels.FsUser.FindUserById(l.ctx, userinfo.UserId)
	if err != nil && !errors.Is(err, gorm.ErrRecordNotFound) {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeServiceErr, "get user info err")
	}
	//获取合适尺寸
	if req.Size > 0 {
		req.Size = image.GetCurrentSize(req.Size)
	}
	//查询分类列表
	tStatus := int64(1)
	tReq := gmodel.GetAllTagByParamsReq{
		Status:    &tStatus,
		OrderBy:   "`sort` DESC",
		WithChild: true, //需要子集
		Category:  1,    //前台网站用的
		Level:     2,    //等级是2的
	}
	//传入分类id
	if req.Cid > 0 {
		//获取该类行的levelprefix
		tagData, err := l.svcCtx.AllModels.FsTags.FindOne(l.ctx, req.Cid)
		if err != nil {
			if errors.Is(err, gorm.ErrRecordNotFound) {
				return resp.SetStatusWithMessage(basic.CodeDbRecordNotFoundErr, "param cid is invalid:record not found")
			}
			logx.Error(err)
			return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get tag info")
		}
		//前台用的分类是1,且等级是2
		if *tagData.Category != 1 && *tagData.Level != 2 {
			return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "invalid tag")
		}
		tReq.LevelPrefixLeftLike = *tagData.LevelPrefix
	}
	tagList, err := l.svcCtx.AllModels.FsTags.GetAllTagByParams(l.ctx, tReq)
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get tag list")
	}
	if len(tagList) == 0 {
		return resp.SetStatusWithMessage(basic.CodeOK, "success", []interface{}{})
	}
	typeIds := make([]int64, 0, len(tagList))
	for _, v := range tagList {
		typeIds = append(typeIds, v.Id)
	}
	var (
		productList                 []gmodel.FsProduct //产品列表（select 字段需要看查询的地方）
		mapTagProp                  = make(map[int64][]types.CoverDefaultItem)
		mapProductHaveOptionFitting = make(map[int64]struct{})
		mapProductMinPrice          = make(map[int64]int64)                   //产品最小价格map
		mapProductSizeCount         = make(map[int64]int64)                   //产品尺寸数量map
		mapProductTemplate          = make(map[int64]int64)                   //产品模板map
		mapResourceMetadata         = make(map[string]map[string]interface{}) //资源元数据
	)
	//携带产品
	if req.WithProduct {
		productList, err = l.getProductRelationInfo(getProductRelationInfoReq{
			Ctx:                         l.ctx,
			TemplateTag:                 req.TemplateTag,
			TypeIds:                     typeIds,
			MapTagProp:                  mapTagProp,
			MapProductHaveOptionFitting: mapProductHaveOptionFitting,
			MapProductMinPrice:          mapProductMinPrice,
			MapProductSizeCount:         mapProductSizeCount,
			MapProductTemplate:          mapProductTemplate,
			MapResourceMetadata:         mapResourceMetadata,
		})
		if err != nil {
			logx.Error(err)
			return resp.SetStatusWithMessage(basic.CodeServiceErr, "failed to get product series info")
		}
	}
	//map tag菜单
	mapTagLevel := make(map[string]*types.TagItem)
	//处理tags数据
	minLevel := 0 //层级最高层（即prefix层级路径最短）
	if err = l.dealWithTagMenuData(dealWithTagMenuDataReq{
		TagList:                     tagList,
		WithProduct:                 req.WithProduct,
		ProductList:                 productList,
		MapTagProp:                  mapTagProp,
		MapProductMinPrice:          mapProductMinPrice,
		MapProductTemplate:          mapProductTemplate,
		MapProductSizeCount:         mapProductSizeCount,
		MapTagLevel:                 mapTagLevel,
		MapProductHaveOptionFitting: mapProductHaveOptionFitting,
		MapResourceMetadata:         mapResourceMetadata,
		Size:                        req.Size,
		User:                        user,
		MinLevel:                    &minLevel,
	}); err != nil {
		logx.Error(err)
		return resp.SetStatusAddMessage(basic.CodeServiceErr, "failed to deal with tag data")
	}
	//组装等级从属关系
	rspTagList, TotalCategoryProduct := l.organizationLevelRelation(minLevel, mapTagLevel)
	return resp.SetStatusWithMessage(basic.CodeOK, "success", types.GetTagProductListRsp{
		TotalCategoryProduct: TotalCategoryProduct,
		TagList:              rspTagList,
	})
}

// 如果携带产品，就查询产品相关信息
type getProductRelationInfoReq struct {
	Ctx                         context.Context
	TemplateTag                 string
	TypeIds                     []int64
	MapTagProp                  map[int64][]types.CoverDefaultItem
	MapProductHaveOptionFitting map[int64]struct{}
	MapProductMinPrice          map[int64]int64
	MapProductSizeCount         map[int64]int64
	MapProductTemplate          map[int64]int64
	MapResourceMetadata         map[string]map[string]interface{}
}

func (l *GetTagProductListLogic) getProductRelationInfo(req getProductRelationInfoReq) (productList []gmodel.FsProduct, err error) {
	var (
		productTemplatesV2List  []gmodel.FsProductTemplateV2
		productSizeCountList    []gmodel.CountProductSizeByStatusRsp
		productOptionalPartList []gmodel.GetGroupPartListByProductIdsRsp
		resourceIds             = make([]string, 0, 200) //涉及资源的id集合
	)
	//查询符合的产品列表
	pIsDel := int64(0)
	pStatus := int64(1)
	pIsShelf := int64(1)
	//获取产品列表
	productList, err = l.svcCtx.AllModels.FsProduct.GetProductListByParams(l.ctx,
		gmodel.GetProductListByParamsReq{
			Type:    req.TypeIds,
			IsDel:   &pIsDel,
			IsShelf: &pIsShelf,
			Status:  &pStatus,
			OrderBy: "`is_recommend` DESC,`sort` ASC",
		})
	if err != nil {
		logx.Error(err)
		return nil, errors.New("failed to get product list")
	}
	productIds := make([]int64, 0, len(productList))
	for _, product := range productList {
		productIds = append(productIds, product.Id)
		resourceIds = append(resourceIds, s3url_to_s3id.GetS3ResourceIdFormUrl(*product.Cover))
		resourceIds = append(resourceIds, s3url_to_s3id.GetS3ResourceIdFormUrl(*product.CoverImg))
	}
	//获取商品可选配件
	productOptionalPartList, err = l.svcCtx.AllModels.FsProductModel3d.GetGroupPartListByProductIds(l.ctx, productIds)
	if err != nil {
		logx.Error(err)
		return nil, errors.New("failed to get product part list")
	}
	//存储有配件的map
	for _, partList := range productOptionalPartList {
		partList.PartList = strings.Trim(partList.PartList, " ")
		partList.PartList = strings.Trim(partList.PartList, ",")
		if partList.PartList == "" {
			continue
		}
		req.MapProductHaveOptionFitting[partList.ProductId] = struct{}{}
	}
	//获取产品标签相关属性
	productTagPropList, err := l.svcCtx.AllModels.FsProductTagProp.GetTagPropByProductIdsWithProductTag(l.ctx, productIds)
	if err != nil {
		logx.Error(err)
		return nil, errors.New("failed to get product tag property")
	}
	for _, v := range productTagPropList {
		resourceIds = append(resourceIds, s3url_to_s3id.GetS3ResourceIdFormUrl(*v.Cover))
	}
	//根据resourceUrls找到对应的元数据
	resourceMetadataList, err := l.svcCtx.AllModels.FsResource.FindAllByResourceIds(l.ctx, resourceIds)
	if err != nil {
		logx.Error(err)
		return nil, errors.New("failed to get resource list")
	}
	for _, v := range resourceMetadataList {
		var metadata map[string]interface{}
		if v.Metadata != nil {
			_ = json.Unmarshal(*v.Metadata, &metadata)
		}
		req.MapResourceMetadata[*v.ResourceUrl] = metadata
	}
	for _, v := range productTagPropList {
		req.MapTagProp[*v.ProductId] = append(req.MapTagProp[*v.ProductId], types.CoverDefaultItem{
			TemplateTag:   v.TemplateTag,
			Cover:         *v.Cover,
			CoverMetadata: req.MapResourceMetadata[*v.Cover],
		})
	}
	//获取产品价格列表
	productPriceList, err := l.svcCtx.AllModels.FsProductPrice.GetSimplePriceListByProductIds(l.ctx, productIds)
	if err != nil {
		logx.Error(err)
		return nil, errors.New("failed to get product min price list")
	}
	//存储产品最小价格
	for _, v := range productPriceList {
		priceStrSlic := strings.Split(v.Price, ",")
		priceSlice, err := format.StrSlicToIntSlice(priceStrSlic)
		if err != nil {
			logx.Error(err)
			return nil, errors.New("parse price err")
		}
		if len(priceSlice) == 0 {
			continue
		}
		//正序排序价格（注意排序后的阶梯价格不能用作阶梯数量价格计算）
		sort.Ints(priceSlice)
		if min, ok := req.MapProductMinPrice[v.ProductId]; ok {
			if min > int64(priceSlice[0]) {
				req.MapProductMinPrice[v.ProductId] = int64(priceSlice[0])
			}
		} else {
			req.MapProductMinPrice[v.ProductId] = int64(priceSlice[0])
		}
	}
	//获取模板
	productTemplatesV2List, err = l.svcCtx.AllModels.FsProductTemplateV2.FindAllByProductIds(l.ctx, productIds, "sort ASC", "product_id,id,model_id,template_tag")
	if err != nil {
		logx.Error(err)
		return nil, errors.New("failed to get product templates")
	}
	for _, v := range productTemplatesV2List {
		//如果已经有了则不用管
		if _, ok := req.MapProductTemplate[*v.ProductId]; ok {
			continue
		} else { //否则设置一个值
			req.MapProductTemplate[*v.ProductId] = v.Id
		}
	}

	//获取产品尺寸数量
	productSizeCountList, err = l.svcCtx.AllModels.FsProductSize.GetGroupProductSizeByStatus(l.ctx, productIds, 1)
	if err != nil {
		logx.Error(err)
		return nil, errors.New("get product size count err")
	}
	for _, v := range productSizeCountList {
		req.MapProductSizeCount[v.ProductId] = v.Num
	}
	return productList, nil
}

// 处理tag菜单数据
type dealWithTagMenuDataReq struct {
	TagList                     []gmodel.FsTags
	WithProduct                 bool
	ProductList                 []gmodel.FsProduct
	MapTagProp                  map[int64][]types.CoverDefaultItem
	ProductTagPropList          []gmodel.FsProductTagProp
	MapProductMinPrice          map[int64]int64
	MapProductTemplate          map[int64]int64
	MapProductSizeCount         map[int64]int64
	MapTagLevel                 map[string]*types.TagItem
	MapProductHaveOptionFitting map[int64]struct{}
	MapResourceMetadata         map[string]map[string]interface{}
	Size                        uint32
	User                        gmodel.FsUser
	MinLevel                    *int //层级最小的
}

func (l *GetTagProductListLogic) dealWithTagMenuData(req dealWithTagMenuDataReq) error {
	for _, tagInfo := range req.TagList {
		prefixSlice := strings.Split(*tagInfo.LevelPrefix, "/")
		lenLevel := len(prefixSlice)
		if *req.MinLevel > lenLevel || *req.MinLevel == 0 {
			*req.MinLevel = lenLevel
		}
		tagTem := types.TagItem{
			TagProductList: []interface{}{},
			TypeName:       *tagInfo.Title,
			TypeId:         tagInfo.Id,
			Icon:           *tagInfo.Icon,
			Sort:           *tagInfo.Sort,
			LevelPrefix:    *tagInfo.LevelPrefix,
			ChildTagList:   make([]*types.TagItem, 0, 50),
		}
		//携带产品
		if req.WithProduct {
			//获取分类产品列表
			productListRsp := l.getTagProducts(getTagProductsReq{
				TagId:                       tagInfo.Id,
				ProductList:                 req.ProductList,
				MapTagProp:                  req.MapTagProp,
				MapProductMinPrice:          req.MapProductMinPrice,
				MapProductTemplate:          req.MapProductTemplate,
				MapProductSizeCount:         req.MapProductSizeCount,
				MapProductHaveOptionFitting: req.MapProductHaveOptionFitting,
				MapResourceMetadata:         req.MapResourceMetadata,
				Size:                        req.Size,
				User:                        req.User,
			})
			//tag中产品
			for _, tmpProduct := range productListRsp {
				tagTem.TagProductList = append(tagTem.TagProductList, tmpProduct)
			}
		}
		//加入分类
		req.MapTagLevel[*tagInfo.LevelPrefix] = &tagTem
	}
	return nil
}

// 组织等级从属关系
func (l *GetTagProductListLogic) organizationLevelRelation(minLevel int, mapTagLevel map[string]*types.TagItem) (rspTagList []types.TagItem, productCount int) {
	mapTop := make(map[string]struct{})
	//设置归属关系
	for prefix, tagItem := range mapTagLevel {
		prefixSlice := strings.Split(prefix, "/")
		//存储最高等级
		if len(prefixSlice) == minLevel {
			mapTop[prefix] = struct{}{}
			continue
		}
		//有父级
		parentPrefix := strings.Join(prefixSlice[:len(prefixSlice)-1], "/")
		parent, ok := mapTagLevel[parentPrefix]
		if !ok {
			continue
		}
		parent.ChildTagList = append(parent.ChildTagList, tagItem)
		//排序
		sort.SliceStable(parent.ChildTagList, func(i, j int) bool {
			return parent.ChildTagList[i].Sort < parent.ChildTagList[j].Sort
		})
		mapTagLevel[parentPrefix] = parent
	}
	//把子类的产品列表变成产品id列表并且把产品列表都放到最顶级父级中
	for _, v := range mapTagLevel {
		if _, ok := mapTop[v.LevelPrefix]; ok {
			continue
		}
		topPrefixSlic := strings.Split(v.LevelPrefix, "/")[:minLevel]
		topPrefix := strings.Join(topPrefixSlic, "/")
		for k, tmpProduct := range v.TagProductList {
			v.TagProductList[k] = tmpProduct.(types.TagProduct).ProductId
			mapTagLevel[topPrefix].TagProductList = append(mapTagLevel[topPrefix].TagProductList, tmpProduct)
		}
	}
	//最终值提取最高级别那一层出来
	rspList := make([]types.TagItem, 0, len(mapTagLevel))
	for prefix, _ := range mapTop {
		//大类下没有任何产品则不显示
		if len(mapTagLevel[prefix].TagProductList) == 0 {
			continue
		}
		productCount += len(mapTagLevel[prefix].TagProductList)
		rspList = append(rspList, *mapTagLevel[prefix])
	}
	//排序
	sort.SliceStable(rspList, func(i, j int) bool {
		return rspList[i].Sort < rspList[j].Sort
	})
	return rspList, productCount
}

// 获取某个tag的直属产品
type getTagProductsReq struct {
	TagId                       int64
	ProductList                 []gmodel.FsProduct
	MapTagProp                  map[int64][]types.CoverDefaultItem
	MapProductMinPrice          map[int64]int64
	MapProductTemplate          map[int64]int64
	MapProductSizeCount         map[int64]int64
	MapProductHaveOptionFitting map[int64]struct{}
	MapResourceMetadata         map[string]map[string]interface{}
	Size                        uint32
	User                        gmodel.FsUser
}

// 返回每个标签下的直接从属产品，不包含子tag的产品
func (l *GetTagProductListLogic) getTagProducts(req getTagProductsReq) (productListRsp []types.TagProduct) {
	//默认给50个容量
	productListRsp = make([]types.TagProduct, 0, 50)
	for _, productInfo := range req.ProductList {
		//不属于一个类型则跳过
		if *productInfo.Type != req.TagId {
			continue
		}
		minPrice, ok := req.MapProductMinPrice[productInfo.Id]
		_, tmpOk := req.MapProductTemplate[productInfo.Id]
		//无最小价格则不显示 || 没有模板也不显示
		if !ok || !tmpOk {
			continue
		}
		sizeNum := int64(0)
		if mapSizeNum, ok := req.MapProductSizeCount[productInfo.Id]; ok {
			sizeNum = mapSizeNum
		}
		//有无可选配件
		haveOptionalFitting := false
		if _, ok = req.MapProductHaveOptionFitting[productInfo.Id]; ok {
			haveOptionalFitting = true
		}
		item := types.TagProduct{
			ProductId:           productInfo.Id,
			Sn:                  *productInfo.Sn,
			Title:               *productInfo.Title,
			SizeNum:             uint32(sizeNum),
			CoverDefault:        []types.CoverDefaultItem{},
			MinPrice:            minPrice,
			HaveOptionalFitting: haveOptionalFitting,
			Recommended:         *productInfo.IsRecommend > 0,
			Cover:               *productInfo.Cover,
			CoverMetadata:       req.MapResourceMetadata[*productInfo.Cover],
			IsCustomization:     *productInfo.IsCustomization,
		}
		if _, ok = req.MapTagProp[productInfo.Id]; ok {
			item.CoverDefault = req.MapTagProp[productInfo.Id]
		}
		//千人千面处理
		/*r := image.ThousandFaceImageFormatReq{
			Size:           int(req.Size),
			IsThousandFace: 0,
			Cover:          *productInfo.Cover,
			CoverImg:       *productInfo.CoverImg,
			ProductId:      productInfo.Id,
			UserId:         req.User.Id,
		}
		if req.User.Id != 0 {
			r.IsThousandFace = int(*req.User.IsThousandFace)
		}
		image.ThousandFaceImageFormat(&r)
		item.Cover = r.Cover*/
		//加入分类产品切片
		productListRsp = append(productListRsp, item)
	}
	return productListRsp
}
